@@ -19,8 +19,9 @@
 #' @param alpha A number specifying the significance level; the default is 0.05.
 #' @param resampling The resampling method to be used, one of "paramBS" (parametric bootstrap
 #'   approach) and "WildBS" (wild bootstrap approach with Rademacher weights).
-#' @param CPU The number of cores used for parallel computing. If omitted, cores are
-#'   detected via \code{\link[parallel]{detectCores}}.
+#' @param para If parallel computing should be used. Default is FALSE.
+#' @param CPU The number of cores used for parallel computing. If not specified, cores
+#'  are detected via \code{\link[parallel]{detectCores}}.
 #' @param seed A random seed for the resampling procedure. If omitted, no 
 #'   reproducible seed is set.
 #' @param dec Number of decimals the results should be rounded to. Default is 3.
@@ -58,7 +59,7 @@
 #' library(tidyr)
 #' eeg <- spread(EEG, feature, resp)
 #' fit <- multRM(cbind(brainrate, complexity) ~ sex * region, data = eeg, 
-#'               subject = "id", within = "region", iter = 200)
+#'               subject = "id", within = "region")
 #' summary(fit)
 #' }
 #' 
@@ -91,21 +92,17 @@
 
 multRM <- function(formula, data, subject, within,
                    iter = 10000, alpha = 0.05, resampling = "paramBS",
-                   CPU, seed, dec = 3){
+                   para = FALSE, CPU, seed, dec = 3){
   
   if (!(resampling %in% c("paramBS", "WildBS"))){
     stop("Resampling must be one of 'paramBS' and 'WildBS'!")
   }
   
-  if(!is.data.frame(data)){
-    data <- as.data.frame(data)
-  }
-  
-  output <- list()
-  
-  test1 <- hasArg(CPU)
-  if(!test1){
-    CPU <- parallel::detectCores()
+  if(para){
+    test1 <- hasArg(CPU)
+    if(!test1){
+      CPU <- parallel::detectCores()
+    }
   }
   
   test2 <- hasArg(seed)
@@ -115,149 +112,23 @@ multRM <- function(formula, data, subject, within,
   
   input_list <- list(formula = formula, data = data,
                      iter = iter, alpha = alpha, resampling = resampling, seed = seed)
-  
-  dat <- model.frame(formula, data)
-  if (!(subject %in% names(data))){
-    stop("The subject variable is not found!")
-  }
-  subject <- data[, subject]
-  if (length(subject) != nrow(dat)){
-    stop("There are missing values in the data.")
-  }
-  
-  dat <- data.frame(dat, subject = subject)
-  nr_hypo <- attr(terms(formula), "factors")
-  perm_names <- t(attr(terms(formula), "factors")[-1, ])
-  fac_names <- colnames(nr_hypo)
-  fac_names_simple <- colnames(perm_names)
-  
-  if(!all(within %in% fac_names)){
-    stop(paste0("The within-subjects factor ", within[which(!(within %in% fac_names_simple))], " is not part of the formula."))
-  }
-  
-  outcome_names <- rownames(nr_hypo)[1]  # names of outcome variables
-  # extract names of outcome variables
-  if (grepl("cbind", outcome_names)){
-    split1 <- strsplit(outcome_names, "(", fixed = TRUE)[[1]][-1]
-    split2 <- strsplit(split1, ")", fixed = TRUE)[[1]]
-    split3 <- strsplit(split2, ",")[[1]]
-  } else {
-    split3 <- outcome_names
-  }
-  
-  EF <- rownames(nr_hypo)[-1]  # names of influencing factors
-  nf <- length(EF)
-  names(dat) <- c("response", EF, "subject")
-  #no. dimensions
-  p <- ncol(as.matrix(dat$response))
-  fl <- NA
-  for (aa in 1:nf) {
-    fl[aa] <- nlevels(as.factor(dat[, (aa + 1)]))
-  }
-  levels <- list()
-  for (jj in 1:nf) {
-    levels[[jj]] <- levels(as.factor(dat[, (jj + 1)]))
-  }
-  lev_names <- expand.grid(levels)
-  # number of hypotheses
-  tmp <- 0
-  for (i in 1:nf) {
-    tmp <- c(tmp, choose(nf, i))
-    nh <- sum(tmp)
-  }
-  
-  names(fl) <- fac_names_simple
-  
-  # determine within- and between-subject factors
-  no.subf <- length(within)
-  no.whole <- nf-no.subf
-  whole <- fac_names_simple[which(!fac_names_simple %in% within)]
-  lev.sub <- prod(fl[within])
-  
-  # correct formula?
-  if (length(fac_names) != nf && length(fac_names) != nh){
-    stop("Something is wrong with the formula. Please specify all or no interactions in crossed designs.")
-  }
-  
-  # check that subjects are correctly labeled
-    if(nrow(data)/length(unique(subject)) != prod(fl[within])){
-      stop(paste0("The number of subjects (", length(unique(subject)), ") times the number of within-subject factor levels
-                  (", prod(fl[within]), ") does not equal the total number of observations (", nrow(data), "). 
-                  There are missing values in the data."))
-    }
-  
-  if (nf == 1) {
-    # one-way layout
-    dat2 <- dat[order(dat[, 2]), ]
-    dat2 <- dat2[order(dat2[, "subject"]), ]
-    #fac.groups <- dat2[, 2]
-    hypo <- list(diag(fl) - matrix(1 / fl, ncol = fl, nrow = fl))
-    Y <- list(dat2)
-    lev.sub <- fl
-    names(fl) <- within
-    # end one-way layout ------------------------------------------------------
-  } else {
-    dat2 <- dat[do.call(order, dat[, c(2:(no.whole+1),      #order whole-plot factors
-                                       ncol(dat),           # order by subject
-                                       ((no.whole+2):(no.whole+2+no.subf-1)))]), ] # order sub-plot factors
-   # dat2 <- dat2[order(dat2[, "subject"]), ]
-    #fac.groups <- do.call(list, dat2[, 2:(nf+1)])
-    lev_names <- lev_names[do.call(order, lev_names[, 1:nf]), ]
-    if(length(whole) ==0){
-      Y <- list(dat2)
-    } else {
-       Y<- split(dat2, dat2[, whole], lex.order = TRUE)
-    }
-  }
-  nind <- sapply(Y, nrow)/lev.sub
-  
-  ## adapting formula argument, if interaction term missing
-  if (nrow(perm_names) != nh) {
-    form2 <- as.formula(paste(outcome_names, "~", paste(fac_names, collapse = "*")))
-    perm_names2 <- t(attr(terms(form2), "factors")[-1, ])
-    fac_names2 <- attr(terms(form2), "term.labels")
-    hyps <- HC(fl, perm_names2, fac_names2)
-    hypo_matrices <- hyps[[1]]
-    fac_names2 <- hyps[[2]]
-    # choose only relevant entries of the hypo matrices
-    indices <- grep(":", fac_names2, invert = TRUE)
-    hypo <- lapply(indices, function(x) hypo_matrices[[x]])
-    
-  } else if(nf !=1){
-    hyp <- HC(fl, perm_names, fac_names)
-    hypo <- hyp[[1]]
-    fac_names <- hyp[[2]]
-  }
-
-  hypo_matrices <- lapply(hypo, function(x) x %x% diag(p))
-  # correcting for "empty" combinations (if no interaction specified)
-  n.groups <- prod(fl[whole])
-  if(nf != 1 & length(Y) != n.groups){
-    index <- NULL
-    for(i in 1:length(Y)){
-      if(nrow(Y[[i]]) == 0){
-        index <- c(index, i)
-      }
-    }
-    Y <- Y[-index]
-  }
-  
-  Ywhole <- lapply(Y, function(x) x$response)
-  if (p==1){
-    Ywhole <- lapply(Ywhole, function(x) as.matrix(x))
-  }
-  
-  Yw2 <- lapply(Ywhole, function(x) matrix(t(x), nrow = nrow(x)/prod(fl[within]), ncol = p*prod(fl[within]), byrow=TRUE))
-  
-  # ---------------------- error detection ------------------------------------
-  # no factor combinations with less than 2 observations
-  if (0 %in% nind || 1 %in% nind) {
-    stop("There is at least one factor-level combination
-           with less than 2 observations!")
-  }
-  
-  #--------------------------------------------------------------------------#
-  
+  #--------------------------------------------------------------------------------#
+  # prepare data for the calculations
+  prepdt <- prepare.data(formula, data, subject, within)
+  # extract relevant info
+  p <- prepdt[["p"]]
+  EF <- prepdt[["EF"]]
+  nf <- prepdt[["nf"]]
+  split3 <- prepdt[["split3"]]
+  no.whole <- prepdt[["no.whole"]]
+  Yw2 <- prepdt[["data"]]
+  hypo_matrices <- prepdt[["hypo"]]
+  fac_names <- prepdt[["fac"]]
+  nind <- prepdt[["n"]]
+  fl <- prepdt[["fl"]]
+  lev_names <- prepdt[["lev_names"]]
+  no.subf <- prepdt[["no.subf"]]
+  #-------------------------------------------------------------------------#
   WTS_out <- matrix(NA, ncol = 3, nrow = length(hypo_matrices))
   WTPS_out <- matrix(NA, nrow = length(hypo_matrices), ncol = 2)
   MATS_out <- matrix(NA, nrow = length(hypo_matrices), ncol = 1)
@@ -270,7 +141,9 @@ multRM <- function(formula, data, subject, within,
   colnames(quantiles) <- c("WTS_resampling", "MATS_resampling")
   # calculate results
   for (i in 1:length(hypo_matrices)) {
-    results <- multRM.Statistic(Yw2, nind, hypo_matrices[[i]], iter, alpha, resampling, CPU, seed, p, t=prod(fl[within]))
+    results <- multRM.Statistic(Yw2, nind, hypo_matrices[[i]], iter, alpha, resampling, 
+                                para, CPU,
+                                seed, p, t=prod(fl[within]))
     WTS_out[i, ] <- round(results$WTS, dec)
     WTPS_out[i, ] <- round(results$WTPS, dec)
     MATS_out[i] <- round(results$MATS, dec)
@@ -287,6 +160,7 @@ multRM <- function(formula, data, subject, within,
   colnames(MATS_out) <- "Test statistic"
   
   # Output ------------------------------------------------------
+  output <- list()
   output$input <- input_list
   output$Descriptive <- descriptive
   output$Covariance <- Var_out
